package org.gcube.dataanalysis.storagehubfacility;

import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.stream.Stream;

import org.gcube.dataanalysis.storagehubfacility.command.StorageHubCommandItemChildren;
import org.gcube.dataanalysis.storagehubfacility.command.StorageHubCommandItemDownload;
import org.gcube.dataanalysis.storagehubfacility.command.StorageHubCommandItemInfo;
import org.gcube.dataanalysis.storagehubfacility.command.StorageHubCommandRootChildren;
import org.gcube.dataanalysis.storagehubfacility.command.StorageHubCommandRootInfo;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * 
 * @author Giancarlo Panichi
 *
 */
public class StorageHubFacility {

	private static Logger logger = LoggerFactory.getLogger(StorageHubFacility.class);

	private static void executeOperation(String operation, String argument1, String gcubeToken, String storageHubUrl,
			String destinationFile) throws Exception {
		switch (operation) {
		case "RootInfo":
			StorageHubCommandRootInfo storageHubCommandRootInfo = new StorageHubCommandRootInfo(gcubeToken, storageHubUrl, destinationFile);
			storageHubCommandRootInfo.execute();
			break;
		case "ItemInfo":
			StorageHubCommandItemInfo storageHubCommandItemInfo = new StorageHubCommandItemInfo(argument1,gcubeToken, storageHubUrl, destinationFile);
			storageHubCommandItemInfo.execute();
			break;
		case "RootChildren":
			StorageHubCommandRootChildren storageHubCommandRootChildren = new StorageHubCommandRootChildren(gcubeToken, storageHubUrl, destinationFile);
			storageHubCommandRootChildren.execute();
			break;
		case "ItemChildren":
			StorageHubCommandItemChildren storageHubCommandItemChildren = new StorageHubCommandItemChildren(argument1,gcubeToken, storageHubUrl, destinationFile);
			storageHubCommandItemChildren.execute();
			break;
		case "Download":
			StorageHubCommandItemDownload storageHubCommandItemDownload = new StorageHubCommandItemDownload(argument1,gcubeToken, storageHubUrl, destinationFile);
			storageHubCommandItemDownload.execute();
		default:
		}

	}

	public static void main(String[] args) {
		try {
			

			logger.debug("StorageHub Facility");
			logger.debug("Operation: " + args[0]);
			logger.debug("Argument1: " + args[1]);
			String destinationFile = "outFile";

			String fileName = "globalvariables.csv";
			String gcubeToken = null;

			try (Stream<String> stream = Files.lines(Paths.get(fileName))) {
				gcubeToken = stream.filter(e -> e.contains("gcube_token")).findFirst().get();
			} catch (Throwable e) {
				logger.error(e.getLocalizedMessage(), e);
				return;
			}

			if (gcubeToken == null || gcubeToken.isEmpty()) {
				logger.error("Token not retrieved in globalvariables.csv file");
				return;
			} else {
				gcubeToken = gcubeToken.substring(14);
				gcubeToken = gcubeToken.replace("\"", "");
				logger.debug("Retrieved GCubeToken");
			}

			ISSupport isSupport = new ISSupport();
			String storageHubUrl = isSupport.discoverStorageHub(gcubeToken);

			executeOperation(args[0], args[1], gcubeToken, storageHubUrl, destinationFile);

		} catch (Throwable e) {
			logger.error("Error in StorageHub Facility: " + e.getLocalizedMessage(), e);
		}
	}

}