package org.gcube.dataanalysis.storagehubfacility.command;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.net.HttpURLConnection;
import java.net.URL;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.nio.file.StandardOpenOption;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * 
 * @author Giancarlo Panichi
 *
 */
public class StorageHubCommandItemChildren implements StorageHubCommand {
	private static Logger logger = LoggerFactory.getLogger(StorageHubCommandItemChildren.class);
	private String itemId;
	private String gcubeToken;
	private String storageHubUrl;
	private String destinationFile;

	public StorageHubCommandItemChildren(String itemId, String gcubeToken, String storageHubUrl,
			String destinationFile) {
		this.itemId = itemId;
		this.gcubeToken = gcubeToken;
		this.storageHubUrl = storageHubUrl;
		this.destinationFile = destinationFile;

	}

	@Override
	public void execute() throws Exception {
		try {
			logger.debug("Execute StorageHubCommandItemChildren");
			logger.debug(storageHubUrl + "/items/" + itemId + "/children?exclude=hl:accounting");
			
			String urlString = storageHubUrl + "/items/" + itemId + "/children?exclude=hl:accounting&gcube-token=" + gcubeToken;

			URL url = new URL(urlString);
			HttpURLConnection conn = (HttpURLConnection) url.openConnection();
			conn.setRequestMethod("GET");
			conn.setRequestProperty("Content-Type", "text/xml; charset=UTF-8");
			try (BufferedInputStream in = new BufferedInputStream(conn.getInputStream())) {
				Path fileOutput = Paths.get(destinationFile);
				try (BufferedOutputStream out = new BufferedOutputStream(
						Files.newOutputStream(fileOutput, StandardOpenOption.CREATE))) {
					byte[] buff = new byte[32 * 1024];
					int len = 0;
					while ((len = in.read(buff)) > 0)
						out.write(buff, 0, len);
				}
			}
		} catch (Exception e) {
			logger.error("StorageHub Facility error in StorageHubCommandItemChildren: " + e.getLocalizedMessage(), e);
			throw e;
		}

	}

}
