package org.gcube.dataanalysis.storagehubfacility.command;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.URL;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.nio.file.StandardOpenOption;

import org.json.JSONObject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * 
 * @author Giancarlo Panichi
 *
 */
public class StorageHubCommandRootChildren implements StorageHubCommand {
	private static Logger logger = LoggerFactory.getLogger(StorageHubCommandRootChildren.class);

	private String gcubeToken;
	private String storageHubUrl;
	private String destinationFile;

	public StorageHubCommandRootChildren(String gcubeToken, String storageHubUrl, String destinationFile) {
		this.gcubeToken = gcubeToken;
		this.storageHubUrl = storageHubUrl;
		this.destinationFile = destinationFile;

	}

	@Override
	public void execute() throws Exception {
		try {
			logger.debug("Execute StorageHubCommandRootChildren");
			logger.debug(storageHubUrl + "/?exclude=hl:accounting");

			String rootId = null;

			String urlString = storageHubUrl + "/?exclude=hl:accounting&gcube-token=" + gcubeToken;

			URL url = new URL(urlString);
			HttpURLConnection conn = (HttpURLConnection) url.openConnection();
			conn.setRequestMethod("GET");
			conn.setRequestProperty("Content-Type", "text/xml; charset=UTF-8");
			try (InputStreamReader in = new InputStreamReader(conn.getInputStream())) {
				try (BufferedReader bin = new BufferedReader(in)) {
					char[] arr = new char[8 * 1024];
					StringBuilder bRootInfo = new StringBuilder();
					int numCharsRead;
					while ((numCharsRead = bin.read(arr, 0, arr.length)) != -1) {
						bRootInfo.append(arr, 0, numCharsRead);
					}

					JSONObject rootInfoJ = new JSONObject(bRootInfo.toString());
					JSONObject itemJ = rootInfoJ.getJSONObject("item");
					rootId = itemJ.getString("id");

				}
			}

			if (rootId == null || rootId.isEmpty()) {
				logger.error("StorageHub Facility error in getItemChildren invalid Root Id: " + rootId);
				return;
			}
			
			logger.debug(storageHubUrl + "/items/" + rootId + "/children?exclude=hl:accounting");
			
			urlString = storageHubUrl + "/items/" + rootId + "/children?exclude=hl:accounting&gcube-token=" + gcubeToken;

			url = new URL(urlString);
			conn = (HttpURLConnection) url.openConnection();
			conn.setRequestMethod("GET");
			conn.setRequestProperty("Content-Type", "text/xml; charset=UTF-8");
			try (BufferedInputStream in = new BufferedInputStream(conn.getInputStream())) {
				Path fileOutput = Paths.get(destinationFile);
				try (BufferedOutputStream out = new BufferedOutputStream(
						Files.newOutputStream(fileOutput, StandardOpenOption.CREATE))) {
					byte[] buff = new byte[32 * 1024];
					int len = 0;
					while ((len = in.read(buff)) > 0)
						out.write(buff, 0, len);
				}
			}

		} catch (Exception e) {
			logger.error("StorageHub Facility error in StorageHubCommandRootChildren: " + e.getLocalizedMessage(), e);
			throw e;
		}

	}

}
